/* Version information.  This file may be modified by scripts.
 *
 * $Id: pgpVersion.h,v 1.1.2.3 1997/07/15 22:24:47 quark Exp $
 *
 */

#define VERSION_OS    "linux"
#define VERSION_MAJOR "5"
#define VERSION_MINOR "0i"
#define VERSION_SUB   "b8"

