/*
* pgpRadix64.h -- The header for Radix64 encoding,
* which is used by ASCII Armor.
*
* Copyright (C) 1995-1997 Pretty Good Privacy, Inc. All rights reserved.
*
* This is a PRIVATE header file, for use only within the PGP Library.
* You should not be using these functions in an application.
*
* $Id: pgpRadix64.h,v 1.3.2.1 1997/06/07 09:50:58 mhw Exp $
*/

#ifndef PGPRADIX64_H
#define PGPRADIX64_H

#ifdef __cplusplus
extern "C" {
#endif

extern char const armorTable[65];

#ifdef __cplusplus
}
#endif

#endif /* PGPRADIX64_H */
