copy config_win32.h config.h
if not exist include mkdir include
if not exist lib\include mkdir lib\include
copy lib\bn\bn.h include
copy lib\bn\bnprime.h include
copy lib\bn\bngermain.h include
copy lib\pgp\cipher\pgpCFB.h include
copy lib\pgp\cipher\pgpCipher.h include
copy lib\pgp\compress\pgpCompress.h include
copy lib\pgp\compress\pgpZInflate.h lib\include
copy lib\pgp\hash\pgpHash.h include
copy lib\pgp\helper\pgpFIFO.h include
copy lib\pgp\helper\pgpPassCach.h include
copy lib\pgp\helper\pgpFile.h include
copy lib\pgp\helper\pgpStr2Key.h include
copy lib\pgp\helper\pgpTimeDate.h include
copy lib\pgp\helper\pgpMem.h include
copy lib\pgp\helper\pgpDebug.h include
copy lib\pgp\helper\pgpLeaks.h include
copy lib\pgp\helper\pgpTypes.h include
copy lib\pgp\helper\pgpFileNames.h include
copy lib\pgp\helper\pgpFileRef.h include
copy lib\pgp\helper\pgpByteFIFO.h lib\include
copy lib\pgp\helper\pgpCharMap.h lib\include
copy lib\pgp\include\pgpAnnotate.h include
copy lib\pgp\include\pgpConvKey.h include
copy lib\pgp\include\pgpErr.h include
copy lib\pgp\include\pgpMsg.h include
copy lib\pgp\include\pgpUI.h include
copy lib\pgp\include\pgpUsuals.h include
copy lib\pgp\include\pgpKludge.h lib\include
copy lib\pgp\include\pgpPktByte.h lib\include
copy lib\pgp\keydb\pgpKeyDB.h include
copy lib\pgp\keydb\pgpPrefUnix.h include
copy lib\pgp\keydb\pgpKDBint.h lib\include
copy lib\pgp\keydb\pgpPrefsInt.h lib\include
copy lib\pgp\keys\pgpRngMnt.h include
copy lib\pgp\keys\pgpRngPub.h include
copy lib\pgp\keys\pgpRngRead.h include
copy lib\pgp\keys\pgpTrust.h include
copy lib\pgp\keys\pgpTrstPkt.h include
copy lib\pgp\keys\pgpRngPriv.h lib\include
copy lib\pgp\keys\pgpRngPkt.h lib\include
copy lib\pgp\keys\pgpRngPars.h lib\include
copy lib\pgp\keys\pgpMemPool.h lib\include
copy lib\pgp\pipe\crypt\pgpCiphrMod.h include
copy lib\pgp\pipe\crypt\pgpConvMod.h include
copy lib\pgp\pipe\crypt\pgpPKEMod.h include
copy lib\pgp\pipe\file\pgpArmor.h include
copy lib\pgp\pipe\file\pgpArmrFil.h include
copy lib\pgp\pipe\file\pgpFileMod.h include
copy lib\pgp\pipe\file\pgpHeader.h include
copy lib\pgp\pipe\file\pgpPrsAsc.h include
copy lib\pgp\pipe\parser\pgpPrsBin.h include
copy lib\pgp\pipe\parser\pgpReadAnn.h include
copy lib\pgp\pipe\parser\pgpVerifyRa.h include
copy lib\pgp\pipe\sig\pgpHashMod.h include
copy lib\pgp\pipe\sig\pgpSigMod.h include
copy lib\pgp\pipe\text\pgpCompMod.h include
copy lib\pgp\pipe\text\pgpLiteral.h include
copy lib\pgp\pipe\text\pgpTextFilt.h include
copy lib\pgp\pipe\utils\pgpBufMod.h include
copy lib\pgp\pipe\utils\pgpDevNull.h include
copy lib\pgp\pipe\utils\pgpJoin.h include
copy lib\pgp\pipe\utils\pgpMemMod.h include
copy lib\pgp\pipe\utils\pgpPipeline.h include
copy lib\pgp\pipe\utils\pgpSplit.h include
copy lib\pgp\pipe\utils\pgpAddHdr.h lib\include
copy lib\pgp\pipe\utils\pgpCopyMod.h lib\include
copy lib\pgp\pubkey\pgpESK.h include
copy lib\pgp\pubkey\pgpFixedKey.h include
copy lib\pgp\pubkey\pgpPubKey.h include
copy lib\pgp\pubkey\pgpSig.h include
copy lib\pgp\pubkey\pgpKeySpec.h include
copy lib\pgp\pubkey\pgpRSAKey.h lib\include
copy lib\pgp\pubkey\pgpMakeSig.h lib\include
copy lib\pgp\pubkey\pgpElGKey.h lib\include
copy lib\pgp\pubkey\pgpDSAKey.h lib\include
copy lib\pgp\pubkey\pgpKeyMisc.h lib\include
copy lib\pgp\random\pgpRndom.h include
copy lib\pgp\random\pgpRndPool.h include
copy lib\pgp\random\pgpRndSeed.h include
copy lib\pgp\utils\pgpConf.h include
copy lib\pgp\utils\pgpDecPipe.h include
copy lib\pgp\utils\pgpEncPipe.h include
copy lib\pgp\utils\pgpFileType.h include
copy lib\pgp\utils\pgpEnv.h include
copy lib\pgp\utils\pgpSigPipe.h include
copy lib\pgp\utils\pgpSigSpec.h include
copy lib\simple\spgp.h include
copy lib\ttyui\pgpUserIO.h include
copy lib\ttyui\pgpRingUI.h include
