/*
* pgpCompress.h -- this defines the compression and decompression
* functions for PGP.. It provides a generalized interface to the
* compression modules, and keeps applications from requiring
* knowledge of all the different types of compression that may be
* supported.
*
* Copyright (C) 1996,1997 Pretty Good Privacy, Inc. All rights reserved.
*
* This is a Public API Function Header.
*
* $Id: pgpCompress.h,v 1.1.2.1 1997/06/07 09:49:56 mhw Exp $
*/

#ifndef PGPCOMPRESS_H
#define PGPCOMPRESS_H

/* Defintions for known compression algorithms */

#define PGP_COMPRESSALG_ZIP 1

#endif /* PGPCOMPRESS_H */
