/*
 * util.c -- Miscellaneous shared code/data
 *
 * Copyright (C) 1997 Pretty Good Privacy, Inc.
 *
 * Written by Mark H. Weaver
 *
 * $Id: util.c,v 1.6 1997/07/07 21:26:41 colin Exp $
 */

char const hexDigits[] = "0123456789abcdef";
char const radix64Digits[] =
#if 0	/* Standard */
	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
#else	/* Modified form that avoids hard-to-OCR characters */
	"ABCDEFGHIJKLMN@PQRSTUVWXYZabcdefghijk\\mn&pqrstuvwxyz0123456789+/";
#endif

/*
 * Local Variables:
 * tab-width: 4
 * End:
 * vi: ts=4 sw=4
 * vim: si
 */
